-----------------------------------------------------------------------------------------------
-- Client Lua Script for FocusUp
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Apollo"
require "Window"
require "Unit"
require "Spell"
require "GameLib"
require "ChatSystemLib"
require "ChatChannelLib"
require "CombatFloater"
require "GroupLib"
 
-----------------------------------------------------------------------------------------------
-- FocusUp Module Definition
-----------------------------------------------------------------------------------------------
local FocusUp = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
focusup_mini_opacity = 0.35;
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function FocusUp:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	isCombat = 0;
	spellTable = {}
	tableRow = 1;
	combatTimer = 0;

    return o
end

function FocusUp:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- FocusUp OnLoad
-----------------------------------------------------------------------------------------------
function FocusUp:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("FocusUp.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- FocusUp OnDocLoaded
-----------------------------------------------------------------------------------------------
function FocusUp:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		Apollo.LoadSprites("FocusUpSpr.xml");
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "FocusUpForm", nil, self)
		self.wndMini = Apollo.LoadForm(self.xmlDoc, "FocusMini", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		self.timer = ApolloTimer.Create(1.0, true, "OnTimer", self)
		Apollo.RegisterSlashCommand("focusup", "OnFocusUpOn", self)
		Apollo.RegisterEventHandler("CombatLogVitalModifier", "OnCombatLogVitalModifier", self)
		Apollo.RegisterEventHandler("UnitEnteredCombat", "OnEnteredCombat", self)
		Apollo.RegisterEventHandler("WindowManagementReady", "OnWindowManagementReady", self)


		-- Do additional Addon initialization here
		self.wndMini:SetOpacity(focusup_mini_opacity);
		self.timer:Stop();
	end
end

-----------------------------------------------------------------------------------------------
-- FocusUp Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/focusup"
function FocusUp:OnFocusUpOn()
	self.wndMain:Invoke() -- show the window
end

function FocusUp:OnTimer()
	--Combat check since it's not fired on reloadui
	if isCombat == 1 then
		combatTimer = combatTimer + 1;
		focusTable = self.wndMain:FindChild("grd_List");
		
		for k,v in pairs(spellTable) do
			focusTable:SetCellText(v.nRow,3,round(v.nValue/combatTimer,2) .. "/s");
			focusTable:SetCellText(v.nRow,2,round(v.nValue/combatTimer*150));
		end
	end
end

function round(num, idp)
	local mult = 10^(idp or 0)
	return math.floor(num * mult + 0.5) / mult
end

-----------------------------------------------------------------------------------------------
-- FocusUpForm Functions
-----------------------------------------------------------------------------------------------
-- when the Cancel button is clicked
function FocusUp:OnCancel()
	self.wndMain:Close() -- hide the window
end

--When a vital is changed
function FocusUp:OnCombatLogVitalModifier(tEventArgs)
	--Find the grid man
	focusTable = self.wndMain:FindChild("grd_List");
	
	--If we're in combat
	if isCombat == 1 then
	
		--Define our variables
		if tEventArgs.eVitalType then
			strVital = Unit.GetVitalTable()[tEventArgs.eVitalType].strName
		end
		
		strValue = tEventArgs.nAmount
		strSpell = tEventArgs.splCallingSpell:GetName()
		strCaster = tEventArgs.unitCaster:GetName()
		strTarget = tEventArgs.unitTarget:GetName()
				
		--Format into table, if the player is the caster and receiver 
		if strCaster == GameLib.GetPlayerUnit():GetName() and strTarget == GameLib.GetPlayerUnit():GetName() and strVital == "Focus" then
			
			if tableRow > 1 and spellTable[strSpell] then
				--Loop reference row number
				local rowNumberLoop = spellTable[strSpell]["nRow"]
			
				--Existing value, update and rewrite
				strValue = strValue + spellTable[strSpell]["nValue"]
				
				spellTable[strSpell] = {
					nRow = rowNumberLoop,
					nValue = strValue,
					nRating = strValue/0.0067
				}
			else			
				--New spell add a row
				focusTable:AddRow(strSpell)
				
				--Capture row number in table
				spellTable[strSpell] = {
					nRow = tableRow,
					nValue = strValue,
					nRating = strValue/0.0067
				}
				
				--Add to row count
				tableRow = tableRow + 1;
			end			
			
		end	
		
	end
end

--When combat event is fired
function FocusUp:OnEnteredCombat(unit, inCombat)
focusTable = self.wndMain:FindChild("grd_List");
	if unit:GetId() == GameLib.GetPlayerUnit():GetId() then
		if inCombat then
			self.timer:Start();
			isCombat = 1;
			--Print("Entering Combat");
			combatTimer = 0;
			for k,v in pairs(spellTable) do spellTable[k]=nil end
			focusTable:DeleteAll();
		else
			isCombat = 0;
			tableRow = 1;
			self.timer:Stop();
			--Print("Leaving Combat");
		end
	end
end

function FocusUp:OnWindowManagementReady()
    Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndMini, strName = "FocusUp"})
	Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndMain, strName = "FocusUpMain"})
end

---------------------------------------------------------------------------------------------------
-- FocusMini Functions
---------------------------------------------------------------------------------------------------

function FocusUp:showMainWindow()
	self.wndMain:Invoke();
end

function FocusUp:mouseEnterMini()
	self.wndMini:SetOpacity(1);
end

function FocusUp:mouseExitMini()
self.wndMini:SetOpacity(focusup_mini_opacity);
end

-----------------------------------------------------------------------------------------------
-- FocusUp Instance
-----------------------------------------------------------------------------------------------
local FocusUpInst = FocusUp:new()
FocusUpInst:Init()
